var grid = new ej.grids.Grid({
        gridLines: 'Both',
        height: '300px',
        dataSource: dataManager,
        editSettings: { allowEditing: true, allowEditOnDblClick: false, allowAdding: true, allowDeleting: true, showDeleteConfirmDialog: true, mode: 'Dialog', template: '#dialogtemplate' },
        showColumnChooser: true,
        allowPaging: true,
        allowSorting: true,
        allowResizing: true,
        allowGrouping: false,
        allowTextWrap: true,
        actionFailure: gridFailure,
        owSelected: onRowSelected,
        filterSettings: { type: 'Menu' },
        pageSettings: {
            pageSize: 50,
            pageSizes: [10, 30, 50, 100, 1000, 5000, 10000]
        },
        toolbar:
            [
                'Add', 'Edit', 'Delete', 'Search',
                { text: 'Reprocessar', tooltipText: 'Reprocessar', prefixIcon: 'e-reload', id: 'Grid_reprocessar' },
                { text: 'Visualizar Processamento', tooltipText: 'Visualizar Processamento', prefixIcon: 'e-search', id: 'Grid_visualizarprocessamento' }
            ],
        toolbarClick: clickHandler,
        searchSettings: { fields: ['descricao', 'empresaNome'] },
        sortSettings: {
            columns: [
                { field: 'id', direction: 'desc' }
            ]
        },
        columns: [
            { field: 'id', headerText: '#', width: '50px', isPrimaryKey: true, isIdentity: true, allowSorting: false },
            { field: 'descricao', headerText: 'Descrição', allowSorting: false, template: '#templatedescricao' },
            { field: 'empresaNome', headerText: 'Empresa', width: '250px', allowSorting: false },
            { headerText: 'Entradas', width: '70px', allowSorting: false, template: '${dataInicioEntradasFormatado} \\n ${dataFimEntradasFormatado}' },
            { headerText: 'Saídas', width: '70px', allowSorting: false, template: '${dataInicioSaidasFormatado} \\n ${dataFimSaidasFormatado}' },
            { field: 'desconsiderarDrawbackSuspensao', headerText: 'Retirar Drawback Suspensão', width: '100px', template: '#templatedesconsiderarDrawbackSuspensao', textAlign: 'center', allowSorting: false },
            { field: 'processamentoId', headerText: 'Processamento', width: '130px', allowSorting: false },
            { field: 'iconStatus', headerText: 'Status', width: '150px', template: '#templateiconStatus', allowSorting: false }
        ],
        actionBegin: onGridActionBegin,
        actionComplete: function (args) {
            var isEdit = args.requestType === 'beginEdit';
            if ((args.requestType === 'beginEdit' || args.requestType === 'add')) {
                var data = args.rowData;

                //Dropdown Tipos de Análise
                dropdownTipoAnalise = new ej.dropdowns.DropDownList({
                    dataSource: tiposAnalise,
                    fields: { text: 'tipo', value: 'id' },
                    change: onDropdownTipoAnaliseChange,
                    enabled: !isEdit,
                    index: 0
                });
                dropdownTipoAnalise.appendTo('#tab-container_tab1_dropTipoAnalise');
                if (isEdit) {
                    var index = tiposAnalise.findIndex(function (tipoAnalise) {
                        return tipoAnalise.id == data.eTipoAnalise;
                    });
                    dropdownTipoAnalise.index = index;
                }

                //Dropdown CFOPS
                dropdownCfop = new ej.dropdowns.DropDownList({
                    dataSource: cfopDataManager,
                    placeholder: 'Cfops para o tipo de análise',
                    fields: { groupBy: 'grupo', text: 'codigo', value: 'id' },
                    itemTemplate: '${codigo} - ${descricao}',
                    actionBegin: onDropdownCfop_ActionBegin
                });
                dropdownCfop.appendTo('#tab-container_tab2_dropCfop');

                // Tabs
                tabObj = new ej.navigations.Tab({
                    items: [
                        { header: { 'text': 'Dados' }, content: '#tab-container_tab1' },
                        { header: { 'text': 'CFOP' }, content: '#tab-container_tab2' },
                        { header: { 'text': 'CNPJs' }, content: '#tab-container_tab3', disabled: true }
                    ]
                });
                tabObj.appendTo('#tab-container');

                // Auto-Complete CNPJ
                autocompleteCnpj = new ej.dropdowns.AutoComplete({
                    change: autocompleteInput,
                    placeholder: 'e.g. CNPJ',
                    enabled: !isEdit
                });
                autocompleteCnpj.appendTo('#tab-container_tab3_autocompleteCnpj_id');

                addButton = new ej.buttons.Button({ cssClass: 'e-info', disabled: true });
                addButton.appendTo('#tab-container_tab3_adicionar');

                removeButton = new ej.buttons.Button({ cssClass: 'e-danger', disabled: true });
                removeButton.appendTo('#tab-container_tab3_remover');

                dropdownPart = new ej.dropdowns.MultiSelect({
                    fields: { text: 'Descricao', value: 'Cnpj' },
                    mode: 'CheckBox'
                });
                dropdownPart.appendTo('#CnpjParticipantes');

                //Validator que é aplicado no form inteiro, porém está sendo criado especificamente para o listbox.
                var options = {
                    customPlacement: (inputElement, error) => {
                        document.getElementById('cnpjParticipanteError').appendChild(error);
                    },
                    validationComplete: (emit) => {
                        if (emit.inputName == "CnpjParticipantes" && emit.status == "failure") {
                            isListBoxValid = false;
                        } else {
                            isListBoxValid = true;
                        }
                    }
                };
                formObject = new ej.inputs.FormValidator('#GridEditForm', options);

                $(document).ready(function () {
                    var source = [];
                    $("#jqxWidget").jqxListBox({ selectedIndex: 3, source: source, width: 250, height: 300 });

                    if (isEdit) {
                        var cnpjParticipantes = data.cnpjParticipantes;
                        if (cnpjParticipantes.lenght !== 0) {
                            for (var i = 0; i < cnpjParticipantes.length; i++) {
                                var cnpjText = cnpjParticipantes[0].replace(/^(\d{2})(\d{3})?(\d{3})?(\d{4})?(\d{2})?/, "$1.$2.$3/$4-$5");
                                var item = { label: cnpjText, value: cnpjParticipantes[0] };
                                $("#jqxWidget").jqxListBox('addItem', item);
                            }
                        }
                        $('#jqxWidget').jqxListBox({ disabled: true });
                    }

                    var cnpjsParticipantesRequest = new ej.base.Ajax('/api/Analise/GetAnaliseParticipantesCnpjs', 'GET', true);
                    cnpjsParticipantesRequest.send().then();
                    cnpjsParticipantesRequest.onSuccess = function (data) {
                        var parsed = JSON.parse(data);
                        parsed.forEach(function (el) {
                            var cnpjText = el.replace(/^(\d{2})(\d{3})?(\d{3})?(\d{4})?(\d{2})?/, "$1.$2.$3/$4-$5");
                            var item = { label: cnpjText, value: el };
                            $("#jqxWidget").jqxListBox('addItem', item);
                        });
                    }

                    $('#jqxWidget').on('change', function (event) {
                        $("#tab-container_tab3_remover").prop('disabled', false);
                    });

                });

                // Convert Widget for the DataInicio field
                dtPickerInicio = new ej.calendars.DatePicker({
                    value: data.dataInicioEntradas,
                    max: moment().endOf('month')._d,
                    start: 'Year',
                    depth: 'Year',
                    format: 'MM/yyyy',
                    change: function (args) {
                        refreshDateRanges();
                    },
                    enabled: !isEdit
                }, args.form.elements.namedItem('f_datainicio'));

                // Convert Widget for the DataInicio field
                dtPickerInicioSaida = new ej.calendars.DatePicker({
                    value: data.dataInicioSaidas,
                    max: moment().endOf('month')._d,
                    start: 'Year',
                    depth: 'Year',
                    format: 'MM/yyyy',
                    change: function (args) {
                        refreshDateRanges();
                    },
                    enabled: !isEdit
                }, args.form.elements.namedItem('f_datainicio_saida'));

                // Convert Widget for the DataFim field
                dtPickerFim = new ej.calendars.DatePicker({
                    value: data.dataFimEntradas,
                    max: moment().endOf('month')._d,
                    start: 'Year',
                    depth: 'Year',
                    format: 'MM/yyyy',
                    change: function (args) {
                        refreshDateRanges();
                    },
                    enabled: !isEdit
                }, args.form.elements.namedItem('f_datafim'));

                dtPickerFimSaida = new ej.calendars.DatePicker({
                    value: data.dataFimSaidas,
                    max: moment().endOf('month')._d,
                    start: 'Year',
                    depth: 'Year',
                    format: 'MM/yyyy',
                    change: function (args) {
                        refreshDateRanges();
                    },
                    enabled: !isEdit
                }, args.form.elements.namedItem('f_datafim_saida'));

                dropdownEmpresaObj = new ej.dropdowns.DropDownList({
                    enabled: empresaDataManager.isDataAvailable,
                    dataSource: empresaDataManager,
                    fields: { text: 'nome', value: 'cnpj' },
                    filterBarPlaceholder: 'Pesquisar',
                    allowFiltering: true,
                    filtering: function (e) {
                        var query = new ej.data.Query();
                        if (e.text !== '') {
                            var predicate = new ej.data.Predicate('nome', 'startswith', e.text, true);
                            predicate = predicate.or('nome', 'contains', e.text, true);
                            query = query.where(predicate);
                        }
                        e.updateData(empresaDataManager, query);
                    },
                    change: function (empresa) {
                        dropdownObjEst.destroy();
                        dropdownObjDrop.destroy();
                        if (empresa.itemData != null) {
                            dropdownObjEst = new ej.dropdowns.MultiSelect({
                                dataSource: estabelecimentoDataManager,
                                query: new ej.data.Query().addParams('raizCnpj', empresa.itemData.cnpj),
                                fields: { text: 'descricao', value: 'cnpj' },
                                itemTemplate: '${cnpj} - ${descricao}',
                                mode: 'CheckBox',
                                showSelectAll: true,
                                showDropDownIcon: true,
                                filterBarPlaceholder: 'Pesquisar',
                                allowCustomValue: true,
                                change: function (estabObj) {
                                    dropdownObjDrop.destroy();
                                    if (estabObj.value != null &&
                                        estabObj.value.length > 0) {
                                        var dropDataManager = new ej.data.DataManager({
                                            url: '/api/Bom/GetDropsByCnpj',
                                            adaptor: new ej.data.UrlAdaptor(),
                                            crossDomain: true
                                        });
                                        dropdownObjDrop = new ej.dropdowns.DropDownList({
                                            dataSource: dropDataManager,
                                            query: new ej.data.Query().addParams('cnpj', estabObj.value[0]),
                                            fields: { text: 'nome', value: 'id' },
                                            filterBarPlaceholder: 'Pesquisar',
                                            allowFiltering: true,
                                            filtering: function (e) {
                                                var query = new ej.data.Query().select(['id', 'nome']);
                                                query = (e.text !== '') ? query.where('nome', 'startswith', e.text, true) : query;
                                                e.updateData(dropDataManager, query);
                                            }
                                        });
                                    } else {
                                        dropdownObjDrop = new ej.dropdowns.DropDownList({
                                            enabled: false
                                        });
                                    }
                                    dropdownObjDrop.appendTo('#DropName');
                                }
                            });
                        } else {
                            dropdownObjEst = new ej.dropdowns.MultiSelect({
                                enabled: false
                            });
                        }
                        dropdownObjDrop = new ej.dropdowns.DropDownList({
                            enabled: false
                        });
                        dropdownObjDrop.appendTo('#DropName');
                        dropdownObjEst.appendTo('#CnpjEstabelecimentos');
                    }
                });
                dropdownEmpresaObj.appendTo('#f_empresa');

                //--------------Carrega dropdown Estabelecimento------------
                dropdownObjEst = new ej.dropdowns.MultiSelect({
                    enabled: false
                });
                dropdownObjEst.appendTo('#CnpjEstabelecimentos');

                //----------Carrega dropdown DROP------------
                dropdownObjDrop = new ej.dropdowns.DropDownList({
                    enabled: false
                });
                dropdownObjDrop.appendTo('#DropName');

                // Set initail Focus
                if (args.requestType === 'beginEdit') {
                    $(".containerNovo").hide();
                    $(".containerEdit").show();
                    args.form.elements.namedItem('f_desconsiderardrawbacksuspencao').focus();
                } else {
                    $(".containerNovo").show();
                    $(".containerEdit").hide();
                    args.form.elements.namedItem('f_descricao').focus();
                }

                //Header Modal
                args.dialog.header = args.requestType === 'beginEdit' ? 'Editar análise' : 'Adicionar Nova análise';

                renderDateSliderEntrada();
                renderDateSliderSaida();

                refreshDateRanges();
            }

            if (args.requestType === 'cancel') {
                dropdownEmpresaObj.destroy();
                dropdownObjEst.destroy();
                dropdownObjDrop.destroy();
            }
        },
    });
    grid.appendTo('#Grid');